;; Generated package description from epoch-view.el  -*- no-byte-compile: t -*-
(define-package "epoch-view" "0.0.1.0.20201201.205202" "Minor mode to visualize epoch timestamps" 'nil :url "https://elpa.gnu.org/packages/epoch-view.html" :keywords '("data" "timestamp" "epoch" "unix") :authors '(("Ted Zlatanov" . "tzz@lifelogs.com")) :maintainer '("Ted Zlatanov" . "tzz@lifelogs.com"))
